@@ -42,11 +42,16 @@ import ssl
 import getpass
 import uuid
 from struct import unpack
-from collections import deque
+from collections import deque, namedtuple
 
 # noinspection PyCompatibility,PyUnresolvedReferences
 from builtins import str
-from six import raise_from, string_types, integer_types
+from six import raise_from, string_types, integer_types, PY2
+
+if PY2:
+    from urlparse import urlparse, parse_qs
+else:
+    from urllib.parse import urlparse, parse_qs
 
 import vertica_python
 from .. import errors
@@ -57,12 +62,15 @@ from ..vertica.messages.frontend_messages import CancelRequest
 from ..vertica.log import VerticaLogging
 
 DEFAULT_HOST = 'localhost'
-DEFAULT_USER = getpass.getuser()
 DEFAULT_PORT = 5433
 DEFAULT_PASSWORD = ''
 DEFAULT_LOG_LEVEL = logging.WARNING
 DEFAULT_LOG_PATH = 'vertica_python.log'
-ASCII = 'ascii'
+try:
+    DEFAULT_USER = getpass.getuser()
+except Exception as e:
+    DEFAULT_USER = None
+    print("WARN: Cannot get the login user name: {}".format(str(e)))
 
 
 def connect(**kwargs):
@@ -70,13 +78,53 @@ def connect(**kwargs):
     return Connection(kwargs)
 
 
+def parse_dsn(dsn):
+    """Parse connection string into a dictionary of keywords and values.
+       Connection string format:
+           vertica://<user>:<password>@<host>:<port>/<database>?k1=v1&k2=v2&...
+    """
+    url = urlparse(dsn)
+    if url.scheme != 'vertica':
+        raise ValueError("Only vertica:// scheme is supported.")
+
+    # Ignore blank/invalid values
+    result = {k: v for k, v in (
+        ('host', url.hostname),
+        ('port', url.port),
+        ('user', url.username),
+        ('password', url.password),
+        ('database', url.path[1:])) if v
+    }
+    for key, value in parse_qs(url.query).items():
+        if key == 'backup_server_node':
+            continue
+        elif key in ('connection_load_balance', 'use_prepared_statements', 'ssl'):
+            lower = value[-1].lower()
+            if lower in ('true', 'on', '1'):
+                result[key] = True
+            elif lower in ('false', 'off', '0'):
+                result[key] = False
+        elif key == 'connection_timeout':
+            result[key] = float(value[-1])
+        elif key == 'log_level' and value[-1].isdigit():
+            result[key] = int(value[-1])
+        else:
+            result[key] = value[-1]
+
+    return result
+
+_AddressEntry = namedtuple('_AddressEntry', ['resolved', 'data'])
+
 class _AddressList(object):
     def __init__(self, host, port, backup_nodes, logger):
         """Creates a new deque with the primary host first, followed by any backup hosts"""
 
         self._logger = logger
 
-        # Format of items in deque: (host, port, is_dns_resolved)
+        # Items in address_deque are _AddressEntry values.
+        #   - when resolved is False, data is (host, port)
+        #   - when resolved is True, data is the 5-tuple from getaddrinfo
+        # This allows for lazy resolution. Seek peek() for more.
         self.address_deque = deque()
 
         # load primary host into address_deque
@@ -105,6 +153,7 @@ class _AddressList(object):
         self._logger.debug('Address list: {0}'.format(list(self.address_deque)))
 
     def _append(self, host, port):
+
         if not isinstance(host, string_types):
             err_msg = 'Host must be a string: invalid value: {0}'.format(host)
             self._logger.error(err_msg)
@@ -127,38 +176,39 @@ class _AddressList(object):
             self._logger.error(err_msg)
             raise ValueError(err_msg)
 
-        self.address_deque.append((host, port, False))
+        self.address_deque.append(_AddressEntry(resolved=False, data=(host, port)))
 
     def push(self, host, port):
-        self.address_deque.appendleft((host, port, False))
+        self.address_deque.appendleft(_AddressEntry(resolved=False, data=(host, port)))
 
     def pop(self):
         self.address_deque.popleft()
 
     def peek(self):
-        # do lazy DNS resolution, return the leftmost DNS-resolved address
+        # do lazy DNS resolution, returning the leftmost socket.getaddrinfo result
         if len(self.address_deque) == 0:
             return None
 
         while len(self.address_deque) > 0:
-            host, port, is_dns_resolved = self.address_deque[0]
-            if is_dns_resolved:
-                # return a resolved address
+            entry = self.address_deque[0]
+            if entry.resolved:
+                # return a resolved sockaddrinfo
                 self._logger.debug('Peek at address list: {0}'.format(list(self.address_deque)))
-                return (host, port)
+                return entry.data
             else:
                 # DNS resolve a single host name to multiple IP addresses
                 self.address_deque.popleft()
+                host, port = entry.data
                 try:
-                    resolved_hosts = socket.getaddrinfo(host, port, socket.AF_INET, socket.SOCK_STREAM)
+                    resolved_hosts = socket.getaddrinfo(host, port, 0, socket.SOCK_STREAM)
                 except Exception as e:
                     self._logger.warning('Error resolving host "{0}" on port {1}: {2}'.format(host, port, e))
                     continue
 
-                # add resolved IP addresses to deque
-                for res in reversed(resolved_hosts):
-                    family, socktype, proto, canonname, sockaddr = res
-                    self.address_deque.appendleft((sockaddr[0], sockaddr[1], True))
+                # add resolved addrinfo (AF_INET and AF_INET6 only) to deque
+                for addrinfo in reversed(resolved_hosts):
+                    if addrinfo[0] in (socket.AF_INET, socket.AF_INET6):
+                        self.address_deque.appendleft(_AddressEntry(resolved=True, data=addrinfo))
 
         return None
 
@@ -181,14 +231,9 @@ class Connection(object):
         self.socket = None
 
         options = options or {}
-        self.options = {key: value for key, value in options.items() if value is not None}
-
-        self.options.setdefault('host', DEFAULT_HOST)
-        self.options.setdefault('port', DEFAULT_PORT)
-        self.options.setdefault('user', DEFAULT_USER)
-        self.options.setdefault('database', self.options['user'])
-        self.options.setdefault('password', DEFAULT_PASSWORD)
-        self.options.setdefault('session_label', _generate_session_label())
+        self.options = parse_dsn(options['dsn']) if 'dsn' in options else {}
+        self.options.update({key: value for key, value in options.items() \
+                             if key != 'dsn' and value is not None})
 
         # Set up connection logger
         logger_name = 'vertica_{0}_{1}'.format(id(self), str(uuid.uuid4())) # must be a unique value
@@ -203,6 +248,19 @@ class Connection(object):
             VerticaLogging.setup_file_logging(logger_name, self.options['log_path'],
                                               self.options['log_level'], id(self))
 
+        self.options.setdefault('host', DEFAULT_HOST)
+        self.options.setdefault('port', DEFAULT_PORT)
+        if 'user' not in self.options:
+            if DEFAULT_USER:
+                self.options['user'] = DEFAULT_USER
+            else:
+                msg = 'Connection option "user" is required'
+                self._logger.error(msg)
+                raise KeyError(msg)
+        self.options.setdefault('database', self.options['user'])
+        self.options.setdefault('password', DEFAULT_PASSWORD)
+        self.options.setdefault('session_label', _generate_session_label())
+
         self.address_list = _AddressList(self.options['host'], self.options['port'],
                                          self.options.get('backup_server_node', []), self._logger)
 
@@ -227,6 +285,9 @@ class Connection(object):
 
     def __exit__(self, type_, value, traceback):
         try:
+            # get the transaction status
+            if not self.closed():
+                self.cursor().flush_to_query_ready()
             # if there's no outstanding transaction, we can simply close the connection
             if self.transaction_status in (None, 'in_transaction'):
                 return
@@ -315,9 +376,8 @@ class Connection(object):
         self.socket = raw_socket
         return self.socket
 
-    def create_socket(self):
-        # Address family IPv6 (socket.AF_INET6) is not supported
-        raw_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+    def create_socket(self, family):
+        raw_socket = socket.socket(family, socket.SOCK_STREAM)
         raw_socket.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
         connection_timeout = self.options.get('connection_timeout')
         if connection_timeout is not None:
@@ -337,13 +397,14 @@ class Connection(object):
                 err_msg = "Bad message size: {0}".format(size)
                 self._logger.error(err_msg)
                 raise errors.MessageError(err_msg)
-            res = BackendMessage.from_type(type_=response, data=raw_socket.recv(size-4))
+            res = BackendMessage.from_type(type_=response, data=raw_socket.recv(size - 4))
             self._logger.debug('<= %s', res)
             host = res.get_host()
             port = res.get_port()
             self._logger.info('Load balancing to host "{0}" on port {1}'.format(host, port))
 
-            socket_host, socket_port = raw_socket.getpeername()
+            peer = raw_socket.getpeername()
+            socket_host, socket_port = peer[0], peer[1]
             if host == socket_host and port == socket_port:
                 self._logger.info('Already connecting to host "{0}" on port {1}. Ignore load balancing.'.format(host, port))
                 return raw_socket
@@ -375,9 +436,9 @@ class Connection(object):
                 else:
                     raw_socket = ssl.wrap_socket(raw_socket)
             except CertificateError as e:
-                raise_from(errors.ConnectionError, e)
+                raise_from(errors.ConnectionError(str(e)), e)
             except SSLError as e:
-                raise_from(errors.ConnectionError, e)
+                raise_from(errors.ConnectionError(str(e)), e)
         else:
             err_msg = "SSL requested but not supported by server"
             self._logger.error(err_msg)
@@ -385,25 +446,31 @@ class Connection(object):
         return raw_socket
 
     def establish_connection(self):
-        addr = self.address_list.peek()
+        addrinfo = self.address_list.peek()
         raw_socket = None
         last_exception = None
 
         # Failover: loop to try all addresses
-        while addr:
+        while addrinfo:
+            (family, socktype, proto, canonname, sockaddr) = addrinfo
             last_exception = None
-            host, port = addr
+
+            # _AddressList filters all addrs to AF_INET and AF_INET6, which both
+            # have host and port as values 0, 1 in the sockaddr tuple.
+            host = sockaddr[0]
+            port = sockaddr[1]
 
             self._logger.info('Establishing connection to host "{0}" on port {1}'.format(host, port))
+
             try:
-                raw_socket = self.create_socket()
-                raw_socket.connect((host, port))
+                raw_socket = self.create_socket(family)
+                raw_socket.connect(sockaddr)
                 break
             except Exception as e:
                 self._logger.info('Failed to connect to host "{0}" on port {1}: {2}'.format(host, port, e))
                 last_exception = e
                 self.address_list.pop()
-                addr = self.address_list.peek()
+                addrinfo = self.address_list.peek()
                 raw_socket.close()
 
         # all of the addresses failed
@@ -482,11 +549,14 @@ class Connection(object):
                 message = BackendMessage.from_type(type_, self.read_bytes(size - 4))
                 self._logger.debug('<= %s', message)
                 self.handle_asynchronous_message(message)
+                # handle transaction status
+                if isinstance(message, messages.ReadyForQuery):
+                    self.transaction_status = message.transaction_status
             except (SystemError, IOError) as e:
                 self.close_socket()
                 # noinspection PyTypeChecker
                 self._logger.error(e)
-                raise_from(errors.ConnectionError, e)
+                raise_from(errors.ConnectionError(str(e)), e)
             if not self.is_asynchronous_message(message):
                 break
         return message
@@ -511,30 +581,6 @@ class Connection(object):
             self._logger.error(msg)
             raise errors.MessageError(msg)
 
-    def process_message(self, message):
-        if isinstance(message, messages.ErrorResponse):
-            raise errors.ConnectionError(message.error_message())
-        elif isinstance(message, messages.BackendKeyData):
-            self.backend_pid = message.pid
-            self.backend_key = message.key
-        elif isinstance(message, messages.ReadyForQuery):
-            self.transaction_status = message.transaction_status
-        elif isinstance(message, messages.CommandComplete):
-            # TODO: I'm not ever seeing this actually returned by vertica...
-            # if vertica returns a row count, set the rowcount attribute in cursor
-            # if hasattr(message, 'rows'):
-            #     self.cursor.rowcount = message.rows
-            pass
-        elif isinstance(message, messages.EmptyQueryResponse):
-            pass
-        elif isinstance(message, messages.CopyInResponse):
-            pass
-        else:
-            raise errors.MessageError("Unhandled message: {0}".format(message))
-
-        # set last message
-        self._cursor._message = message
-
     def __str__(self):
         safe_options = {key: value for key, value in self.options.items() if key != 'password'}
 
@@ -554,19 +600,18 @@ class Connection(object):
         return results
 
     def startup_connection(self):
-        # This doesn't handle Unicode usernames or passwords
-        user = self.options['user'].encode(ASCII)
-        database = self.options['database'].encode(ASCII)
-        password = self.options['password'].encode(ASCII)
-        session_label = self.options['session_label'].encode(ASCII)
+        user = self.options['user']
+        database = self.options['database']
+        session_label = self.options['session_label']
+        os_user_name = DEFAULT_USER if DEFAULT_USER else ''
+        password = self.options['password']
 
-        self.write(messages.Startup(user, database, session_label))
+        self.write(messages.Startup(user, database, session_label, os_user_name))
 
         while True:
             message = self.read_message()
 
             if isinstance(message, messages.Authentication):
-                # Password message isn't right format ("incomplete message from client")
                 if message.code == messages.Authentication.OK:
                     self._logger.info("User {} successfully authenticated"
                         .format(self.options['user']))
@@ -582,8 +627,15 @@ class Connection(object):
                                                  {'user': user,
                                                   'salt': getattr(message, 'salt', None),
                                                   'usersalt': getattr(message, 'usersalt', None)}))
-            else:
-                self.process_message(message)
-
-            if isinstance(message, messages.ReadyForQuery):
+            elif isinstance(message, messages.BackendKeyData):
+                self.backend_pid = message.pid
+                self.backend_key = message.key
+            elif isinstance(message, messages.ReadyForQuery):
                 break
+            elif isinstance(message, messages.ErrorResponse):
+                self._logger.error(message.error_message())
+                raise errors.ConnectionError(message.error_message())
+            else:
+                msg = "Received unexpected startup message: {0}".format(message)
+                self._logger.error(msg)
+                raise errors.MessageError(msg)
