@@ -1,4 +1,4 @@
-# Copyright (c) 2018 Micro Focus or one of its affiliates.
+# Copyright (c) 2018-2019 Micro Focus or one of its affiliates.
 # Copyright (c) 2018 Uber Technologies, Inc.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -38,6 +38,7 @@ from __future__ import print_function, division, absolute_import
 
 import datetime
 import re
+from uuid import UUID
 
 try:
     from collections import OrderedDict  # python 2.7+ / 3
@@ -57,8 +58,6 @@ from ..vertica import messages
 from ..vertica.column import Column
 
 
-UTF_8 = 'utf-8'
-
 if six.PY2:
     # noinspection PyUnresolvedReferences
     file_type = (IOBase, file)
@@ -72,9 +71,10 @@ RE_NAME = u'(("{0}")|({0}))'.format(RE_NAME_BASE)
 RE_BASIC_INSERT_STAT = (
     u"INSERT\\s+INTO\\s+(?P<target>({0}\\.)?{0})"
     u"\\s*\\(\\s*(?P<variables>{0}(\\s*,\\s*{0})*)\\s*\\)"
-    u"\\s+VALUES\\s*\\(\\s*(?P<values>.*)\\s*\\)").format(RE_NAME)
+    u"\\s+VALUES\\s*\\(\\s*(?P<values>(.|\\s)*)\\s*\\)").format(RE_NAME)
 END_OF_RESULT_RESPONSES = (messages.CommandComplete, messages.PortalSuspended)
 
+
 class Cursor(object):
     # NOTE: this is used in executemany and is here for pandas compatibility
     _insert_statement = re.compile(RE_BASIC_INSERT_STAT, re.U | re.I)
@@ -116,7 +116,7 @@ class Cursor(object):
 
     def close(self):
         self._logger.info('Close the cursor')
-        if not self.closed():
+        if not self.closed() and self.prepared_sql:
             self._close_prepared_statement()
         self._closed = True
 
@@ -147,7 +147,7 @@ class Cursor(object):
             # If the SQL has not been prepared, prepare the SQL
             if operation != self.prepared_sql:
                 self._prepare(operation)
-                self.prepared_sql = operation # the prepared statement is kept
+                self.prepared_sql = operation  # the prepared statement is kept
 
             # Bind the parameters and execute
             self._execute_prepared_statement([parameters])
@@ -182,7 +182,7 @@ class Cursor(object):
             # If the SQL has not been prepared, prepare the SQL
             if operation != self.prepared_sql:
                 self._prepare(operation)
-                self.prepared_sql = operation # the prepared statement is kept
+                self.prepared_sql = operation  # the prepared statement is kept
 
             # Bind the parameters and execute
             self._execute_prepared_statement(seq_of_parameters)
@@ -200,9 +200,12 @@ class Cursor(object):
                                  for parameters in seq_of_parameters]
                 data = "\n".join(seq_of_values)
 
+                copy_autocommit = self.connection.parameters.get('auto_commit', 'on')
+
                 copy_statement = (
                     u"COPY {0} ({1}) FROM STDIN DELIMITER ',' ENCLOSED BY '\"' "
-                    u"ENFORCELENGTH ABORT ON ERROR").format(target, variables)
+                    u"ENFORCELENGTH ABORT ON ERROR{2}").format(target, variables,
+                    " NO COMMIT" if copy_autocommit == 'off' else '')
 
                 self.copy(copy_statement, data)
             else:
@@ -316,7 +319,6 @@ class Cursor(object):
         while True:
             message = self.connection.read_message()
             if isinstance(message, messages.ReadyForQuery):
-                self.connection.transaction_status = message.transaction_status
                 self._message = message
                 break
 
@@ -336,7 +338,7 @@ class Cursor(object):
 
     def copy(self, sql, data, **kwargs):
         """
-        
+
         EXAMPLE:
         >> with open("/tmp/file.csv", "rb") as fs:
         >>     cursor.copy("COPY table(field1,field2) FROM STDIN DELIMITER ',' ENCLOSED BY ''''",
@@ -359,6 +361,7 @@ class Cursor(object):
         else:
             raise TypeError("Not valid type of data {0}".format(type(data)))
 
+        self._logger.info(u'Execute COPY statement: [{}]'.format(sql))
         self.connection.write(messages.Query(sql))
 
         while True:
@@ -367,14 +370,15 @@ class Cursor(object):
             self._message = message
             if isinstance(message, messages.ErrorResponse):
                 raise errors.QueryError.from_error_response(message, sql)
-
-            self.connection.process_message(message=message)
-
-            if isinstance(message, messages.ReadyForQuery):
+            elif isinstance(message, messages.ReadyForQuery):
                 break
             elif isinstance(message, messages.CopyInResponse):
                 self.connection.write(messages.CopyStream(stream, **kwargs))
                 self.connection.write(messages.CopyDone())
+            elif isinstance(message, messages.CommandComplete):
+                pass
+            else:
+                raise errors.MessageError('Unexpected message: {0}'.format(message))
 
         if self.error is not None:
             raise self.error
@@ -415,9 +419,9 @@ class Cursor(object):
                     key = str(key)
                 key = as_text(key)
 
-                if isinstance(param, string_types):
+                if isinstance(param, (string_types, bytes)):
                     param = self.format_quote(as_text(param), is_csv)
-                elif isinstance(param, (datetime.datetime, datetime.date, datetime.time)):
+                elif isinstance(param, (datetime.datetime, datetime.date, datetime.time, UUID)):
                     param = self.format_quote(as_text(str(param)), is_csv)
                 elif param is None:
                     param = '' if is_csv else NULL
@@ -433,9 +437,9 @@ class Cursor(object):
         elif isinstance(parameters, (tuple, list)):
             tlist = []
             for param in parameters:
-                if isinstance(param, string_types):
+                if isinstance(param, (string_types, bytes)):
                     param = self.format_quote(as_text(param), is_csv)
-                elif isinstance(param, (datetime.datetime, datetime.date, datetime.time)):
+                elif isinstance(param, (datetime.datetime, datetime.date, datetime.time, UUID)):
                     param = self.format_quote(as_text(str(param)), is_csv)
                 elif param is None:
                     param = '' if is_csv else NULL
@@ -511,7 +515,7 @@ class Cursor(object):
         self._message = self.connection.read_expected_message(
                         (messages.RowDescription, messages.NoData), self._error_handler)
         if isinstance(self._message, messages.NoData):
-            self.description = None # response was NoData for a DDL/transaction PreparedStatement
+            self.description = None  # response was NoData for a DDL/transaction PreparedStatement
         else:
             self.description = [Column(fd, self.unicode_error) for fd in self._message.fields]
 
